
#import <Foundation/Foundation.h>
#import "iCore.h"

@class B4I;
@class B4IList;

/**
*BleManager allows you to discover, connect and read data from BLE peripheral devices.
* This library is supported by iOS 7.1+.
*/
//~shortname:BleManager
//~version:2.00
//~Event:StateChanged (State As Int)
//~Event:DeviceFound (Name As String, Id As String, AdvertisingData As Map, RSSI As Double)
//~Event:Disconnected
//~Event:Connected (Services As List)
//~Event:DataAvailable (Service As String, Characteristics As Map)
//~Event:WriteComplete (Characteristic As String, Success As Boolean)
//~ObjectWrapper: CBCentralManager*
@interface BleManager : B4IObjectWrapper
@property (nonatomic, readonly) int STATE_UNKNOWN;
@property (nonatomic, readonly) int STATE_RESETTING;
@property (nonatomic, readonly) int STATE_UNSUPPORTED;
@property (nonatomic, readonly) int STATE_UNAUTHORIZED;
@property (nonatomic, readonly) int STATE_POWERED_OFF;
@property (nonatomic, readonly) int STATE_POWERED_ON;
/**
 * Returns the current state.
 */
@property (nonatomic, readonly) int State;
/**
 * Returns the characteristic that was last updated. This property can be used inside the DataAvailable event to find which
 *characteristic was last updated (the Map will include all the characteristics).
 */
@property (nonatomic, readonly) NSString *LastUpdatedCharacteristic;
/**
* Adds or removes a notification listener that monitors value changes.
*Note that you must call ReadData once before you can call this method.
*The DataAvailable method will be raised when the value of the characteristic changes.
*Service - The service id (as returned in the Connected event).
*Characteristic - The characteristic id (as returned in the DataAvailable event).
*Notify - True to add a listener, false to remove it.
*/
- (void)SetNotify:(NSString *)Service :(NSString *)Characteristic :(BOOL)Notify;

/**
* Initializes the object. The StateChanged event will be raised after this method with the current BLE state.
*/
- (void)Initialize:(B4I *)bi :(NSString *)EventName;

/**
* Starts scanning for devices. DeviceFound event will be raised when a device is found.
*ServiceUUIDs - A list (or array) with service uuids. Devices that don't advertise these uuids will not be discovered.
* Pass Null to discover all devices.
*/
- (void)Scan:(B4IList *)ServiceUUIDs;

/**
* Similar to Scan. If AllowDuplicates is true then the DeviceFound event will be raised whenever a packet is received.
*/
- (void)Scan2:(B4IList *)ServiceUUIDs :(bool)AllowDuplicates;

/**
* Stops scanning for new devices.
*/
- (void)StopScan;

/**
* Connects to a device with the given id. You can only connect to previously discovered devices.
* Note that the Disconnected event will be raised if the connection has failed.
*/
- (void)Connect:(NSString *)DeviceId;
//Disconnects a connected device.
- (void)Disconnect;

/**
* Asynchronously reads all characteristics from the given service. The DataAvailable will be raised when the data is available.
*/
- (void)ReadData:(NSString *)Service;

/**
* Asynchronously reads the value of the specified characteristic. Note that you must call ReadData once before you can call this method.
*The DataAvailable will be raised when the data of this characteristic is available.
*/
- (void)ReadData2:(NSString *)Service :(NSString *)Characteristic;

/**
* Writes the data to the specified characteristic.
*You must call ReadData once and wait for the DataAvailable event before you can use this method.
*Note that this method does not raise any event.
*Service - The service id (as returned in the Connected event).
*Characteristic - The characteristic id (as returned in the DataAvailable event).
*Data - Data to write.
*/
- (void)WriteData:(NSString *)Service :(NSString *)Characteristic :(B4IArray */*byte,1*/)Data;
/**
 * Similar to WriteData. Writes the data and expects a response.
 */
- (void)WriteDataWithResponse:(NSString *)Service :(NSString *)Characteristic :(B4IArray */*byte,1*/)Data;
/**
 * Returns a numeric value from which you can find the properties of the specified characteristic.
 */
- (int)GetCharacteristicProperties:(NSString *)Service :(NSString *)Characteristic;
/**
 * Returns the connected peripheral object. To be used with NativeObject.
 */
- (NSObject *)GetPeripheralObject;
//Returns the service object. To be used with NativeObject.
- (NSObject *)GetServiceObject:(NSString *)Service;
//Returns the characteristic object. To be used with NativeObject.
- (NSObject *)GetCharacteristicObject:(NSString *)ServiceId :(NSString *)CharacteristicId;
@end
//~ObjectWrapper: CBPeripheralManager*
//~Event:StateChanged (State As Int)
//~Event:Subscribe (CentralId As String)
//~Event: Unsubscribe(CentralId As String)
//~Event: NewData (Data() As Byte)
//~shortname: PeripheralManager
@interface B4IPeripheralManager : B4IObjectWrapper
@property (nonatomic, readonly) int STATE_UNKNOWN;
@property (nonatomic, readonly) int STATE_RESETTING;
@property (nonatomic, readonly) int STATE_UNSUPPORTED;
@property (nonatomic, readonly) int STATE_UNAUTHORIZED;
@property (nonatomic, readonly) int STATE_POWERED_OFF;
@property (nonatomic, readonly) int STATE_POWERED_ON;
/**
 * Returns the current state.
 */
@property (nonatomic, readonly) int State;
/**
 * Initializes the object and set the subs that will handle the events.
 */
- (void)Initialize:(B4I *)bi :(NSString *)EventName;
/**
 * Starts advertising and listening for connections.
 *Name - The peripheral name (will be advertised).
 */
- (void)Start:(NSString *)Name;
/**
 * Writes data to the subscribed centrals via a notification.
 *Centrals - An array or list with the subscribed central ids. Pass Null to send to all.
 *Data - Data to send. Maximum size is 20 bytes.
 */
- (void)Write:(B4IList *)Centrals :(B4IArray */*byte,1*/)Data;
/**
 * Stops advertising.
 */
- (void)Stop;
@end
